@@ -36,7 +36,7 @@ except ImportError:
     print("  $ sudo easy_install paramiko")
     exit(1)
 
-__version__ = '0.0.3'
+__version__ = '0.0.4'
 __author__ = 'Christian Vest Hansen'
 __author_email__ = 'karmazilla@gmail.com'
 __url__ = 'https://savannah.nongnu.org/projects/fab/'
@@ -51,7 +51,7 @@ __greeter__ = '''\
 ENV = {
     'fab_version':__version__,
     'fab_author':__author__,
-    'fab_mode':'fanout',
+    'fab_mode':'rolling',
     'fab_port':22,
     'fab_user':None,
     'fab_pkey':None,
@@ -59,7 +59,7 @@ ENV = {
     'fab_new_host_key':'accept',
     'fab_shell':'/bin/bash -l -c "%s"',
     'fab_timestamp':datetime.datetime.utcnow().strftime('%F_%H-%M-%S'),
-    'fab_debug':False,
+    'fab_print_real_sudo':False,
 }
 
 CONNECTIONS = []
@@ -91,6 +91,8 @@ def run_per_host(op_fn):
         if not CONNECTIONS:
             _connect()
         _on_hosts_do(op_fn, *args, **kvargs)
+    wrapper.__doc__ = op_fn.__doc__
+    wrapper.__name__ = op_fn.__name__
     return wrapper
 
 #
@@ -274,7 +276,7 @@ def sudo(host, client, env, cmd, **kvargs):
     """
     cmd = _lazy_format(cmd, env)
     real_cmd = env['fab_shell'] % ("sudo -S " + cmd.replace('"', '\\"'))
-    cmd = env['fab_debug'] and real_cmd or cmd
+    cmd = env['fab_print_real_sudo'] and real_cmd or cmd
     if not _confirm_proceed('sudo', host, kvargs):
         return False # TODO: should we return False in fail??
     print("[%s] sudo: %s" % (host, cmd))
@@ -782,7 +784,7 @@ def _list_objs(objs):
     for name, fn in cmds:
         print '  ', name.ljust(max_name_len),
         if fn.__doc__:
-            print ':', fn.__doc__.splitlines()[0]
+            print ':', filter(None, fn.__doc__.splitlines())[0].strip()
         else:
             print
 
@@ -804,18 +806,20 @@ def _connect():
         ENV['fab_password'] = getpass.getpass()
     else:
         print("Warning: Putting your password in a fabfile is a bad idea.")
-    port = int(ENV['fab_port'])
+    def_port = ENV['fab_port']
     username = ENV['fab_user']
     password = ENV['fab_password']
     pkey = ENV['fab_pkey']
     key_filename = ENV['fab_key_filename']
     for host in ENV['fab_hosts']:
+        host, _, port = host.partition(':')
+        portnr = int(port or def_port)
         client = ssh.SSHClient()
         client.load_system_host_keys()
         if 'fab_new_host_key' in ENV and ENV['fab_new_host_key'] == 'accept':
             client.set_missing_host_key_policy(ssh.AutoAddPolicy())
         client.connect(
-            host, port, username, password, pkey, key_filename
+            host, portnr, username, password, pkey, key_filename
         )
         CONNECTIONS.append((host, client))
     if not CONNECTIONS:
@@ -832,6 +836,8 @@ def _disconnect():
 
 def _lazy_format(string, env=ENV):
     "Do recursive string substitution of ENV vars - both lazy and earger."
+    if string is None:
+        return None
     def replacer_fn(match):
         var = match.group('var')
         if var in env:
@@ -902,58 +908,56 @@ def _pick_fabfile():
     else:
         return guesses[0] # load() will barf for us...
 
+def _load_default_settings():
+    "Load user-default fabric settings from ~/.fabric"
+    # TODO: http://mail.python.org/pipermail/python-list/2006-July/393819.html
+    cfg = os.path.expanduser("~/.fabric")
+    if os.path.exists(cfg):
+        comments = lambda s: s and not s.startswith("#")
+        settings = filter(comments, open(cfg, 'r'))
+        settings = [(k.strip(),v.strip()) for k,_,v in
+            [s.partition('=') for s in settings]]
+        ENV.update(settings)
+
 def _parse_args(args):
     cmds = []
     for cmd in args:
-        cmd_name = cmd
-        cmd_args = None
-        if cmd.find(':') != -1:
-            cmd_name, cmd_args = cmd.split(':', 1)
-        if cmd_args is not None:
-            cmd_arg_kvs = {}
-            for cmd_arg_kv in cmd_args.split(','):
+        cmd_args = {}
+        if ':' in cmd:
+            cmd, cmd_str_args = cmd.split(':', 1)
+            for cmd_arg_kv in cmd_str_args.split(','):
                 k, _, v = cmd_arg_kv.partition('=')
-                cmd_arg_kvs[k] = (v % ENV)
-            cmd_args = cmd_arg_kvs
-        cmds.append((cmd_name, cmd_args))
+                cmd_args[k] = (v % ENV)
+        cmds.append((cmd, cmd_args))
     return cmds
 
-def _validate_commands(args):
-    for cmd in args:
-        if cmd.find(':') != -1:
-            cmd = cmd.split(':', 1)[0]
-        if not cmd in COMMANDS:
-            print("No such command: %s" % cmd)
-            _list_commands()
-            exit(1)
-    if not args:
+def _validate_commands(cmds):
+    if not cmds:
         print("No commands given.")
         _list_commands()
+    else:
+        for cmd in cmds:
+            if not cmd[0] in COMMANDS:
+                print("No such command: %s" % cmd[0])
+                exit(1)
 
-def _execute_commands(args):
-    for cmd in args:
-        cmd_name = cmd
-        cmd_args = None
-        if cmd.find(':') != -1:
-            cmd_name, cmd_args = cmd.split(':', 1)
-        ENV['fab_cur_command'] = cmd_name
-        print("Running %s..." % cmd_name)
-        if cmd_args is not None:
-            cmd_arg_kvs = {}
-            for cmd_arg_kv in cmd_args.split(','):
-                k, _, v = cmd_arg_kv.partition('=')
-                cmd_arg_kvs[k] = (v % ENV)
-            COMMANDS[cmd_name](**cmd_arg_kvs)
-        else:
-            COMMANDS[cmd]()
+def _execute_commands(cmds):
+    for cmd, args in cmds:
+        ENV['fab_cur_command'] = cmd
+        print("Running %s..." % cmd)
+        if args is not None:
+            args = dict(zip(args.keys(), map(_lazy_format, args.values())))
+        COMMANDS[cmd](**(args or {}))
 
 def main(args):
     try:
         print(__greeter__ % ENV)
         fabfile = _pick_fabfile()
-        load(fabfile)
-        _validate_commands(args)
-        _execute_commands(args)
+        _load_default_settings()
+        load(fabfile, fail='warn')
+        commands = _parse_args(args)
+        _validate_commands(commands)
+        _execute_commands(commands)
     finally:
         _disconnect()
         print("Done.")
